using System;
using System.Collections;
using System.Collections.Generic;

namespace TSLab.Script.Handlers
{
    [HandlerName("Constant")]
    [HandlerCategory("TradeMath")]
    public class ConstGen : IBar2DoubleHandler, IDouble1CalculatorHandler, IDouble2DoubleHandler, IList<double>
    {
        public class Enumerator : IEnumerator<double>
        {
            private readonly int m_size;
            private int m_cur;
            private readonly double m_value;

            public Enumerator(int size, double value)
            {
                m_size = size;
                m_cur = 0;
                m_value = value;
            }
            #region Implementation of IDisposable

            public void Dispose()
            {
            }

            #endregion

            #region Implementation of IEnumerator

            public bool MoveNext()
            {
                return ++m_cur <= m_size;
            }

            public void Reset()
            {
                m_cur = 0;
            }

            public double Current
            {
                get { return m_value; }
            }

            object IEnumerator.Current
            {
                get { return Current; }
            }

            #endregion
        }

        [HandlerParameter]
        public double Value
        {
            get;
            set;
        }

        private int m_count = int.MaxValue;

        private void MakeList(int count)
        {
            m_count = count;
        }

        public IList<double> Execute(ISecurity source)
        {
            MakeList(source.Bars.Count);
            return this;
        }

        public double Execute(double source1)
        {
            return Value;
        }

        public IList<double> Execute(IList<double> source)
        {
            MakeList(source.Count);
            return this;
        }

        #region Implementation of IEnumerable

        public IEnumerator<double> GetEnumerator()
        {
            return new Enumerator(m_count, Value);
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        #endregion

        #region Implementation of ICollection<double>

        public void Add(double item)
        {
            throw new InvalidOperationException();
        }

        public void Clear()
        {
            throw new InvalidOperationException();
        }

        public bool Contains(double item)
        {
// ReSharper disable CompareOfFloatsByEqualityOperator
            return item == Value;
// ReSharper restore CompareOfFloatsByEqualityOperator
        }

        public void CopyTo(double[] array, int arrayIndex)
        {
            throw new InvalidOperationException();
        }

        public bool Remove(double item)
        {
            throw new InvalidOperationException();
        }

        public int Count
        {
            get { return m_count; }
        }

        public bool IsReadOnly
        {
            get { return true; }
        }

        #endregion

        #region Implementation of IList<double>

        public int IndexOf(double item)
        {
            return 0;
        }

        public void Insert(int index, double item)
        {
            throw new InvalidOperationException();
        }

        public void RemoveAt(int index)
        {
            throw new InvalidOperationException();
        }

        public double this[int index]
        {
            get { return Value; }
            set { throw new InvalidOperationException(); }
        }

        #endregion
    }
}