﻿using System.Collections.Generic;
using TSLab.Script.Helpers;

namespace TSLab.Script.Handlers
{
    [HandlerCategory("Indicators")]
    public class StochK : BasePeriodIndicatorHandler, IBar2DoubleHandler, IContextUses
    {
        public IList<double> Execute(ISecurity source)
        {
            var high = Context.GetData("Highest", new[] { Period.ToString(), source.CacheName },
                                       () => Series.Highest(source.HighPrices, Period));
            var low = Context.GetData("Lowest", new[] { Period.ToString(), source.CacheName },
                                       () => Series.Lowest(source.LowPrices, Period));
            var closes = source.ClosePrices;
            var list = new double[closes.Count];
            for (int i = 0; i < closes.Count; i++)
            {
                var hl = high[i] - low[i];
                var stochK = hl == 0 ? 0 : 100 * (closes[i] - low[i]) / hl;
                list[i] = stochK;
            }
            return list;
        }

        public IContext Context { get; set; }
    }

    [HandlerCategory("Indicators")]
    [HandlerName("StochRSI")]
    public class StochRSI : BasePeriodIndicatorHandler, IBar2DoubleHandler, IContextUses
    {
        public IList<double> Execute(ISecurity source)
        {
            var rsi = Context.GetData("RSI", new[] { Period.ToString(), source.CacheName },
                                       () => Series.RSI(source.ClosePrices, Period));
            var high = Series.Highest(rsi, Period);
            var low = Series.Lowest(rsi, Period);
            var list = new double[rsi.Count];
            for (int i = 0; i < rsi.Count; i++)
            {
                var hl = high[i] - low[i];
                var stochRSI = hl == 0 ? 0 : 100 * (rsi[i] - low[i]) / hl;
                list[i] = stochRSI;
            }
            return list;
        }

        public IContext Context { get; set; }
    }

    //[HandlerCategory("Indicators")]
    //[HandlerName("StochRSI (double)")]
    //public class StochRSIDouble : BasePeriodIndicatorHandler, IDouble2DoubleHandler, IContextUses
    //{
    //    public IList<double> Execute(IList<double> source)
    //    {
    //        var rsi = Series.RSI(source, Period);
    //        var high = Series.Highest(Series.RSI(rsi, Period), Period);
    //        var low = Series.Lowest(Series.RSI(rsi, Period), Period);
    //        var list = new double[rsi.Count];
    //        for (int i = 0; i < rsi.Count; i++)
    //        {
    //            var hl = high[i] - low[i];
    //            var stochRSI = hl == 0 ? 0 : 100 * (rsi[i] - low[i]) / hl;
    //            list[i] = stochRSI;
    //        }
    //        return list;
    //    }

    //    public IContext Context { get; set; }
    //}
}
