﻿using System.Collections.Generic;
using TSLab.Script.Helpers;

namespace TSLab.Script.Handlers
{
    [HandlerCategory("TradeMath")]
    public class Flip : IBoolAccum
    {
        public IList<bool> Execute(IList<bool> source1, IList<bool> source2)
        {
            return Calc(source1, source2);
        }

        public bool m_last;

        public bool Execute(bool source1, bool source2)
        {
            m_last = Calc(source1, source2, m_last);
            return m_last;
        }

        public static IList<bool> Calc(IList<bool> source1, IList<bool> source2)
        {
            var res = new bool[source1.Count];
            for (int i = 1; i < source1.Count; i++)
            {
                var last = i > 0 && res[i - 1];
                res[i] = Calc(source1[i], source2[i], last);
            }
            return res;
        }

        public static bool Calc(bool source1, bool source2, bool last)
        {
            var res = last;
            if (!res & source1)
            {
                res = true;
            }
            if (res & source2)
            {
                res = false;
            }
            return res;
        }
    }

    [HandlerCategory("TradeMath")]
    public class Hold : BasePeriodIndicatorHandler, IBoolConvertor
    {
        public IList<bool> Execute(IList<bool> source)
        {
            return Calc(source, Period);
        }

        private int m_savedNum = -1;

        public bool Execute(bool source, int num)
        {
            return Calc(source, Period, num, ref m_savedNum);
        }

        public static IList<bool> Calc(IList<bool> source, int period)
        {
            var res = new bool[source.Count];
            int savedNum = -1;
            for (int i = 1; i < source.Count; i++)
            {
                res[i] = Calc(source[i], period, i, ref savedNum);
            }
            return res;
        }

        public static bool Calc(bool source, int period, int num, ref int savedNum)
        {
            if (source)
            {
                savedNum = num;
            }
            if (savedNum >= 0 && (num - savedNum) <= period)
            {
                source = true;
            }
            return source;
        }
    }

    [HandlerCategory("TradeMath")]
    public class Shift : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Series.Shift(source, Period);
        }
    }
}
