﻿using System.Collections.Generic;
using TSLab.Script.Helpers;

namespace TSLab.Script.Handlers
{
    [HandlerCategory("Indicators")]
    public class TEMA : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Calc(source, Period);
        }

        public static IList<double> Calc(IList<double> source, int period)
        {
            var ema1 = Series.EMA(source, period);
            var ema2 = Series.EMA(ema1, period);
            var ema3 = Series.EMA(ema2, period);
            var res = new double[source.Count];
            for (int i = 0; i < source.Count; i++)
            {
                res[i] = 3 * ema1[i] - 3 * ema2[i] + ema3[i];
            }
            return res;
        }
    }

    [HandlerCategory("Indicators")]
    public class DEMA : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            var ema1 = Series.EMA(source, Period);
            var ema2 = Series.EMA(ema1, Period);
            var res = new double[source.Count];
            for (int i = 0; i < source.Count; i++)
            {
                res[i] = 2 * ema1[i] - ema2[i];
            }
            return res;
        }
    }

    [HandlerCategory("Indicators")]
    [HandlerName("TEMA (Zero Lag)")]
    [HandlerName("TEMA (без задержки)", Language = "ru-RU")]
    public class ZeroLagTEMA : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Calc(source, Period);
        }

        public static IList<double> Calc(IList<double> source, int period)
        {
            var tma1 = TEMA.Calc(source, period);
            var tma2 = TEMA.Calc(tma1, period);
            var res = new double[source.Count];
            for (int i = 0; i < source.Count; i++)
            {
                res[i] = 2 * tma1[i] - tma2[i];
            }
            return res;
        }
    }
}
